@@ -1,7 +1,8 @@
 # File nrmse.R
-# Part of the hydroGOF R package, http://www.rforge.net/hydroGOF/ ; 
-#                                 http://cran.r-project.org/web/packages/hydroGOF/
-# Copyright 2008-2017 Mauricio Zambrano-Bigiarini
+# Part of the hydroGOF R package, https://github.com/hzambran/hydroGOF
+#                                 https://cran.r-project.org/package=hydroGOF
+#                                 http://www.rforge.net/hydroGOF/ ;
+# Copyright 2008-2023 Mauricio Zambrano-Bigiarini
 # Distributed under GPL 2 or later
 
 ################################################################################
@@ -12,6 +13,7 @@
 # Started: 15-Dic-2008                                                         #
 # Updates: 06-Sep-2009                                                         #
 #          03-Jul-2017                                                         #
+#          16-Jan-2023                                                         #
 ################################################################################
 # 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
 # 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
@@ -25,20 +27,36 @@
 
 nrmse <-function(sim, obs, ...) UseMethod("nrmse")
  
-nrmse.default <- function (sim, obs, na.rm=TRUE, norm="sd", ...) {
+nrmse.default <- function(sim, obs, na.rm=TRUE, norm="sd", fun=NULL, ...,
+                          epsilon.type=c("none", "Pushpalatha2012", "otherFactor", "otherValue"), 
+                          epsilon.value=NA) {
 
-    # Checking that the user provied a valid argument for 'norm'       
-    if (is.na(match(norm, c("sd", "maxmin") ) ) ) 
+  # Checking that the user provied a valid argument for 'norm'       
+  if (is.na(match(norm, c("sd", "maxmin") ) ) ) 
        stop("Invalid argument: 'norm' must be in c('sd', 'maxmin')")
 
-    if ( is.na(match(class(sim), c("integer", "numeric", "ts", "zoo", "xts"))) |
-          is.na(match(class(obs), c("integer", "numeric", "ts", "zoo", "xts")))
+  if ( is.na(match(class(sim), c("integer", "numeric", "ts", "zoo", "xts"))) |
+       is.na(match(class(obs), c("integer", "numeric", "ts", "zoo", "xts")))
      ) stop("Invalid argument type: 'sim' & 'obs' have to be of class: c('integer', 'numeric', 'ts', 'zoo', 'xts')")      
 
-    vi <- valindex(sim, obs)
-     
+  epsilon.type <- match.arg(epsilon.type)  
+
+  # index of those elements that are present both in 'sim' and 'obs' (NON- NA values)
+  vi <- valindex(sim, obs)
+   
+  if (length(vi) > 0) {	 
+    # Filtering 'obs' and 'sim', selecting only those pairs of elements 
+    # that are present both in 'x' and 'y' (NON- NA values)
     obs <- obs[vi]
     sim <- sim[vi]
+
+    if (!is.null(fun)) {
+      fun1 <- match.fun(fun)
+      new  <- preproc(sim=sim, obs=obs, fun=fun1, ..., 
+                      epsilon.type=epsilon.type, epsilon.value=epsilon.value)
+      sim  <- new[["sim"]]
+      obs  <- new[["obs"]]
+    } # IF end     
        
     if (norm=="sd") {
       cte <- sd(obs, na.rm=na.rm)
@@ -48,18 +66,20 @@ nrmse.default <- function (sim, obs, na.rm=TRUE, norm="sd", ...) {
 
      rmse <- rmse(sim, obs, na.rm) 
      
-     if (max(obs, na.rm= na.rm) - min(obs, na.rm= na.rm) != 0) {
-     
-       nrmse <- rmse / cte
-     
+     if (max(obs, na.rm= na.rm) - min(obs, na.rm= na.rm) != 0) {     
+       nrmse <- rmse / cte     
      } else {
          nrmse <- NA
-         warning("'obs' is constant, it is not possible to compute 'nrmse'")  
+         warning("'obs' is constant -> it is not possible to compute 'nrmse' !")  
        } # ELSE end
+   } else {
+         nrmse <- NA
+         warning("There are no pairs of 'sim' and 'obs' without missing values !")
+     } # ELSE end
      
-     return( round( 100*nrmse, 1) )
+   return( round( 100*nrmse, 1) )
      
-  } # 'nrmse.default' end
+} # 'nrmse.default' end
   
  
 ################################################################################
@@ -70,7 +90,9 @@ nrmse.default <- function (sim, obs, na.rm=TRUE, norm="sd", ...) {
 # Started: 15-Dic-2008                                                         #
 # Updates: 06-Sep-2009 ; 05-Nov-2012                                           #
 ################################################################################
-nrmse.matrix <- function (sim, obs, na.rm=TRUE, norm="sd", ...) {
+nrmse.matrix <- function(sim, obs, na.rm=TRUE, norm="sd", fun=NULL, ...,
+                         epsilon.type=c("none", "Pushpalatha2012", "otherFactor", "otherValue"), 
+                         epsilon.value=NA) {
 
   # Checking that 'sim' and 'obs' have the same dimensions
   if ( all.equal(dim(sim), dim(obs)) != TRUE )
@@ -85,7 +107,10 @@ nrmse.matrix <- function (sim, obs, na.rm=TRUE, norm="sd", ...) {
   nrmse <- rep(NA, ncol(obs))       
           
   nrmse <- sapply(1:ncol(obs), function(i,x,y) { 
-                 nrmse[i] <- nrmse.default( x[,i], y[,i], na.rm=na.rm, norm=norm, ... )
+                 nrmse[i] <- nrmse.default( x[,i], y[,i], na.rm=na.rm, norm=norm, 
+                                            fun=fun, ..., 
+                                            epsilon.type=epsilon.type,  
+                                            epsilon.value=epsilon.value)
                }, x=sim, y=obs )    
                      
   names(nrmse) <- colnames(obs)
@@ -103,12 +128,15 @@ nrmse.matrix <- function (sim, obs, na.rm=TRUE, norm="sd", ...) {
 # Started: 15-Dic-2008                                                         #
 # Updates: 06-Sep-2009                                                         #
 ################################################################################
-nrmse.data.frame <- function (sim, obs, na.rm=TRUE, norm="sd", ...) {
+nrmse.data.frame <- function(sim, obs, na.rm=TRUE, norm="sd", fun=NULL, ...,
+                             epsilon.type=c("none", "Pushpalatha2012", "otherFactor", "otherValue"), 
+                             epsilon.value=NA) {
 
   sim <- as.matrix(sim)
   obs <- as.matrix(obs)
    
-  nrmse.matrix(sim, obs, na.rm=na.rm, norm=norm, ...)
+  nrmse.matrix(sim, obs, na.rm=na.rm, norm=norm, fun=fun, ..., 
+               epsilon.type=epsilon.type, epsilon.value=epsilon.value)
      
 } # 'nrmse.data.frame' end
   
@@ -119,7 +147,9 @@ nrmse.data.frame <- function (sim, obs, na.rm=TRUE, norm="sd", ...) {
 # Started: 22-Mar-2013                                                         #
 # Updates:                                                                     #
 ################################################################################
-nrmse.zoo <- function(sim, obs, na.rm=TRUE, norm="sd", ...){
+nrmse.zoo <- function(sim, obs, na.rm=TRUE, norm="sd", fun=NULL, ...,
+                      epsilon.type=c("none", "Pushpalatha2012", "otherFactor", "otherValue"), 
+                      epsilon.value=NA){
     
     sim <- zoo::coredata(sim)
     if (is.zoo(obs)) obs <- zoo::coredata(obs)
@@ -128,4 +158,4 @@ nrmse.zoo <- function(sim, obs, na.rm=TRUE, norm="sd", ...){
        nrmse.matrix(sim, obs, na.rm=na.rm, norm=norm, ...)
     } else NextMethod(sim, obs, na.rm=na.rm, norm=norm, ...)
      
-  } # 'nrmse.zoo' end
+} # 'nrmse.zoo' end
